%%%%%%%%%%%%%%%%%%%%%%%%
%%% In-sample Simulation
%%%%%%%%%%%%%%%%%%%%%%%%
%% Load model 
[m,p,mss] = readmodel(false);

%% Time span
rtime = qq(1999,1); %start of sample

stime = qq(2002,1); %starting point of the first simulation 
                    %stime is the initial state
                    %stime+1 is the first simulated time period
etime = qq(2010,1); %the end of the known history
       
%% Selection of historical time series for computing model's forecasting
%properties:
list_xnames = {'dot_cpi','dot4_cpi','rmc','lgdp_gap','ls'};

list_titles = {'Inflation q-o-q','Inflation y-o-y','RMC Gap','Output Gap','Nominal Exchange Rate'};

%% Database preparation
h = loadstruct('kalm_his.mat');
f = dbsub(h, list_xnames);

for i = 1:length(list_xnames)
    list_out{i} = ['out_' list_xnames{i}];
end
           
for i = 1:length(list_xnames)
    eval([list_out{i} '= resize(h.(list_xnames{i}), rtime:etime);'])
end

%% Remove all residuals from the database
list_   = get(m,'enames');
list_db = dbobjects(h);

for i = 1:length(list_)
    if strmatch(list_{i}, list_db)
        h.(list_{i}) = fill(h.(list_{i}), get(h.(list_{i}),'first2last'), zeros(size(get(h.(list_{i}),'first2last'))));
    else
        disp(['Missing residual variable in database: ', list_{i}]);
    end
end

%% Simulations ... (done by a "loop")
%Beginning of the "loop" ...
for time = stime:1:etime
    fcastrng = time:time+8;
    if fcastrng(end)>etime
        exorng = time:etime;
    else
        exorng = time:time+8;
    end

disp(' ');
disp('---------------------------------------------------------------------')
disp(['The first simulated time period of this projection round: ',char(dat2str(time+1))]);
disp(' ');

% Plan for "targets", "foreign variables", etc ...
fcast_plan  = plan(m, fcastrng);
    
% fcast_plan  = exogenize( fcast_plan, 'target', exorng);
% fcast_plan  = endogenize(fcast_plan, 'shk_target', exorng);
 
fcast_plan  = exogenize( fcast_plan, 'lx_gdp_gap', exorng);
fcast_plan  = endogenize(fcast_plan, 'shk_lx_gdp_gap', exorng);

fcast_plan  = exogenize( fcast_plan, 'x_rn', exorng);
fcast_plan  = endogenize(fcast_plan, 'shk_x_rn', exorng);

fcast_plan  = exogenize( fcast_plan, 'dot_x_cpi', exorng);
fcast_plan  = endogenize(fcast_plan, 'shk_dot_x_cpi', exorng);

%Simulate the model
dbfcast = simulate(m,h,fcastrng,'plan',fcast_plan,'anticipate',true);
s = dboverlay(h,dbfcast);
    
%Store the results
list   = dbobjects(s);
range_ = fcastrng(1)-1:fcastrng(end);
s = dbbatch(s, '$0', 'resize(s.$0, range_)', 'namelist', list);
    
for i = 1:length(list_xnames)
    eval([ list_out{i} '= [' list_out{i} ', s.(list_xnames{i})];'])
    end

end %end of the simulation "loop"

%% Graphs
disp('Creating report ...');

srep = qq(2003,1); % beginning of report
erep = qq(2010,4); % end

x = report('heading', 'Bulgaria: In-sample Simulations', 'dateformat', 'YY:P', 'color', ...
             true, 'visible', true);

for i = 1:length(list_xnames)
    if i ~= 1
       x = newpage(x);
    end
    x = opengraph(x);
    eval([ 'p = plot(srep:erep,' list_out{i} ');'])
    title(list_titles{i});
    set(p(1),'linewidth',4);
    for j = 2:length(p)
        set(p(j),'linestyle',':','linewidth',2,'color','r');
    end
    hold on
    grid on
    x = getgraph(x);
end

for i = 1:length(list_xnames)
    if mod(i,4)== 0
        x = newpage(x);
    else
        x = skip(x);
    end
    eval(['[M, me, med, s, err2] = revival(' list_out{i} ', stime:etime, 6);']);
    rownames_ = {'Mean','Median','Std.','Err2'};
    for jj = 1:length(me)
        colnames_{jj} = ['t+' num2str(jj)];
    end

    x = matrix(x, [me; med; s; err2],'text', list_titles {i}, ...
                'rownames', rownames_, 'colnames',colnames_,'decimal', 2);
end

%% Saves all graphs
%% The default is set to generate both .ps and .pdf files:
compile(x,'In-sample.ps','display',false);
compile(x,'In-sample.pdf','display',false);
disp('Done !');